package uk.kerb.sammy.pop
{
   import Box2D.Collision.*;
   import Box2D.Collision.Shapes.*;
   import Box2D.Common.*;
   import Box2D.Common.Math.*;
   import Box2D.Dynamics.*;
   import Box2D.Dynamics.Contacts.*;
   import Box2D.Dynamics.Joints.*;
   import com.memecounter.Tracker;
   import flash.display.*;
   import flash.events.Event;
   import flash.events.MouseEvent;
   import flash.filters.BlurFilter;
   import flash.geom.Point;
   import flash.net.URLRequest;
   import flash.net.navigateToURL;
   import flash.utils.getTimer;
   import uk.kerb.utils.AssetFactory;
   import uk.kerb.utils.sound.SoundManager;
   
   public class Game extends AbstractSection
   {
      
      private static const PENALTY_POINTS:Number = -100;
      
      private static const GAME_DURATION:int = 180000;
      
      private static const TARGET_TOP:Number = 370;
      
      private static const GREEN_TARGET_MAX:Number = 550;
      
      private static const YELLOW_TARGET_MAX:Number = 275;
      
      private static const SCREEN_WIDTH:int = 550;
      
      private static const BLACK_PENALTY_POINTS:Number = -300;
      
      private static const SCREEN_HEIGHT:int = 400;
      
      private static const BLUE_TARGET_MAX:Number = 413;
      
      private static const ZERO_POINT:Point = new Point(0,0);
      
      private static const POINTS:Array = [100,200,300,400,500];
      
      private static const RED_TARGET_MAX:Number = 138;
      
      private static const RAD_TO_DEG:Number = 180 / Math.PI;
      
      private static const ITERATIONS:int = 5;
      
      private static const TIME_STEP:Number = 1 / 20;
       
      
      private var lines:Array;
      
      private var upgradeTimes:Array;
      
      private var score:int;
      
      private var mc:MovieClip;
      
      private var world:b2World;
      
      private var drawing:Boolean;
      
      private var ballSprite:Sprite;
      
      private var goodBandMembers:Array;
      
      private var pointsIndex:int;
      
      private var mx:Number;
      
      private var numEvilBalls:int;
      
      private var my:Number;
      
      private var deadLine_bmp:BitmapData;
      
      private var startTime:int;
      
      private var maxBalls:int;
      
      private var onscreen_bmp:BitmapData;
      
      private var balls:Array;
      
      private var deadLineSprite:Sprite;
      
      private var badBandMembers:Array;
      
      private var offscreen_bmp:BitmapData;
      
      private var liveLineSprite:Sprite;
      
      private var scoreSprite:Sprite;
      
      private var deadLineBlur:BlurFilter;
      
      public function Game()
      {
         super();
      }
      
      private function addToBadBandMembers(param1:Array) : void
      {
         var _loc2_:MovieClip = null;
         var _loc3_:int = 0;
         _loc3_ = 0;
         while(_loc3_ < param1.length)
         {
            _loc2_ = mc.getChildByName("band" + param1[_loc3_] + "_mc") as MovieClip;
            if(badBandMembers.indexOf(_loc2_) == -1)
            {
               badBandMembers.push(goodBandMembers.splice(goodBandMembers.indexOf(_loc2_),1)[0]);
               _loc2_.gotoAndStop("bad");
            }
            _loc3_++;
         }
      }
      
      private function redraw() : void
      {
         var _loc1_:int = 0;
         var _loc2_:Line = null;
         var _loc3_:b2Body = world.m_bodyList;
         while(_loc3_)
         {
            if(_loc3_.m_userData is Sprite)
            {
               _loc3_.m_userData.x = _loc3_.m_position.x;
               _loc3_.m_userData.y = _loc3_.m_position.y;
               _loc3_.m_userData.rotation = _loc3_.m_rotation * (180 / Math.PI);
            }
            _loc3_ = _loc3_.m_next;
         }
         deadLine_bmp.applyFilter(deadLine_bmp,deadLine_bmp.rect,ZERO_POINT,deadLineBlur);
         deadLine_bmp.scroll(0,4);
         liveLineSprite.graphics.clear();
         liveLineSprite.graphics.lineStyle(Line.HAIR_THICKNESS,Line.LIVE_LINE_COLOUR);
         deadLineSprite.graphics.clear();
         deadLineSprite.graphics.lineStyle(Line.HAIR_THICKNESS,Line.DEAD_LINE_COLOUR);
         _loc1_ = int(lines.length);
         while(--_loc1_ > -1)
         {
            _loc2_ = lines[_loc1_];
            if(_loc2_.isDead())
            {
               _loc2_.draw(deadLineSprite.graphics);
               _loc2_.destroy(world);
               lines.splice(_loc1_,1);
            }
            else
            {
               _loc2_.draw(liveLineSprite.graphics);
            }
         }
         deadLine_bmp.draw(deadLineSprite);
         offscreen_bmp.copyPixels(deadLine_bmp,deadLine_bmp.rect,ZERO_POINT);
         offscreen_bmp.draw(liveLineSprite);
         onscreen_bmp.copyPixels(offscreen_bmp,offscreen_bmp.rect,ZERO_POINT);
      }
      
      override public function init(param1:AssetFactory) : void
      {
         super.init(param1);
         mc = param1.createDisplayObject("GameSection") as MovieClip;
         addChild(mc);
         liveLineSprite = new Sprite();
         deadLineSprite = new Sprite();
         deadLineBlur = new BlurFilter(3,5,1);
         deadLine_bmp = new BitmapData(SCREEN_WIDTH,SCREEN_HEIGHT,true,0);
         offscreen_bmp = new BitmapData(SCREEN_WIDTH,SCREEN_HEIGHT,true,0);
         onscreen_bmp = new BitmapData(SCREEN_WIDTH,SCREEN_HEIGHT,true,0);
         var _loc2_:Bitmap = new Bitmap(onscreen_bmp);
         mc.addChild(_loc2_);
         mc.musicCredit_mc.addEventListener(MouseEvent.CLICK,onClickMusicCredit);
         mc.musicCredit_mc.buttonMode = true;
         ballSprite = new Sprite();
         mc.addChild(ballSprite);
         scoreSprite = new Sprite();
         mc.addChild(scoreSprite);
         score = 0;
         maxBalls = 1;
         numEvilBalls = 0;
         upgradeTimes = [30000,75000,120000];
         pointsIndex = 0;
         drawing = false;
         lines = new Array();
         balls = new Array();
         goodBandMembers = new Array(mc.band0_mc,mc.band1_mc,mc.band2_mc,mc.band3_mc);
         badBandMembers = new Array();
         initWorld();
         initGround();
         initWalls();
         initTargetDividers();
         mc.setChildIndex(_loc2_,mc.numChildren - 1);
         mc.setChildIndex(ballSprite,mc.numChildren - 1);
         mc.setChildIndex(scoreSprite,mc.numChildren - 1);
         mc.setChildIndex(mc.band0_mc,mc.numChildren - 1);
         mc.setChildIndex(mc.band1_mc,mc.numChildren - 1);
         mc.setChildIndex(mc.band2_mc,mc.numChildren - 1);
         mc.setChildIndex(mc.band3_mc,mc.numChildren - 1);
         mc.setChildIndex(mc.targets_mc,mc.numChildren - 1);
         mc.setChildIndex(mc.musicCredit_mc,mc.numChildren - 1);
         mc.setChildIndex(mc.panel_mc,mc.numChildren - 1);
         addEventListener(Event.ADDED_TO_STAGE,onAddedToStage);
      }
      
      private function updateBandMembers() : void
      {
         var _loc1_:* = false;
         var _loc2_:int = 0;
         var _loc3_:MovieClip = null;
         var _loc4_:String = null;
         if(goodBandMembers.length > 0)
         {
            _loc1_ = goodBandMembers[0].animation_mc.currentFrame == goodBandMembers[0].animation_mc.totalFrames;
         }
         else
         {
            _loc1_ = true;
         }
         if(_loc1_)
         {
            _loc4_ = "good" + Math.floor(Math.random() * 3);
            _loc2_ = 0;
            while(_loc2_ < badBandMembers.length)
            {
               _loc3_ = badBandMembers[_loc2_];
               if(_loc3_.animation_mc != null && _loc3_.animation_mc.currentFrame == _loc3_.animation_mc.totalFrames)
               {
                  goodBandMembers.push(_loc3_);
                  _loc3_.gotoAndStop(_loc4_);
                  badBandMembers.splice(_loc2_,1);
               }
               _loc2_++;
            }
            _loc2_ = 0;
            while(_loc2_ < goodBandMembers.length)
            {
               goodBandMembers[_loc2_].gotoAndStop(_loc4_);
               _loc2_++;
            }
         }
      }
      
      private function onAddedToStage(param1:Event) : void
      {
         removeEventListener(Event.ADDED_TO_STAGE,onAddedToStage);
         startTime = getTimer();
         stage.addEventListener(MouseEvent.MOUSE_DOWN,onMouseDown);
         stage.addEventListener(MouseEvent.MOUSE_UP,onMouseUp);
         addEventListener(Event.ENTER_FRAME,onEnterFrame);
      }
      
      private function showScoreIcon(param1:int, param2:Ball) : void
      {
         var _loc3_:Number = Number(param2.body.m_userData.x);
         var _loc4_:Number = param2.body.m_userData.y - 20;
         var _loc5_:ScoreMessage = new ScoreMessage(_loc3_,_loc4_,param1,assetFactory);
         scoreSprite.addChild(_loc5_);
      }
      
      private function onEnterFrame(param1:Event) : void
      {
         world.Step(TIME_STEP,ITERATIONS);
         checkBalls();
         if(drawing)
         {
            addLine();
         }
         redraw();
         updateBandMembers();
         updateDisplay();
      }
      
      private function addLine() : void
      {
         if(mx != mouseX && my != mouseY)
         {
            lines.push(new Line(mx,my,mouseX,mouseY,world));
            mx = mouseX;
            my = mouseY;
         }
      }
      
      private function initGround() : void
      {
         var _loc1_:b2BoxDef = new b2BoxDef();
         _loc1_.extents.Set(0.5 * SCREEN_WIDTH,25);
         _loc1_.friction = 0;
         _loc1_.restitution = 1;
         var _loc2_:b2BodyDef = new b2BodyDef();
         _loc2_.position.Set(0.5 * SCREEN_WIDTH,SCREEN_HEIGHT + 25);
         _loc2_.AddShape(_loc1_);
         _loc2_.userData = new Object();
         world.CreateBody(_loc2_);
      }
      
      private function updateDisplay() : void
      {
         var _loc2_:int = 0;
         var _loc3_:int = 0;
         var _loc4_:String = null;
         var _loc5_:String = null;
         mc.panel_mc.score_txt.text = score.toString();
         var _loc1_:int = GAME_DURATION - (getTimer() - startTime);
         if(_loc1_ < 0)
         {
            mc.panel_mc.time_txt.text = "00:00";
            gameOver();
         }
         else
         {
            _loc2_ = _loc1_ / 60000;
            _loc3_ = (_loc1_ - _loc2_ * 60000) / 1000;
            _loc4_ = _loc2_ < 10 ? "0" + _loc2_ : _loc2_.toString();
            _loc5_ = _loc3_ < 10 ? "0" + _loc3_ : _loc3_.toString();
            mc.panel_mc.time_txt.text = _loc4_ + ":" + _loc5_;
         }
      }
      
      override public function getNextSection() : Class
      {
         return GameOver;
      }
      
      private function addBall() : void
      {
         balls.push(new Ball(world,ballSprite,assetFactory));
      }
      
      private function checkBalls() : void
      {
         var _loc2_:Ball = null;
         var _loc3_:b2Body = null;
         var _loc4_:Number = NaN;
         var _loc5_:Number = NaN;
         var _loc6_:Number = NaN;
         var _loc8_:int = 0;
         var _loc1_:int = int(balls.length);
         var _loc7_:Number;
         if((_loc7_ = (getTimer() - startTime) / GAME_DURATION) < 1)
         {
            if(balls.length == 0 || balls.length - numEvilBalls < maxBalls && Math.random() < 0.03)
            {
               addBall();
            }
            if(Math.random() < 0.01 * (getTimer() - startTime) / GAME_DURATION)
            {
               addEvilBall();
            }
         }
         if(upgradeTimes.length > 0 && getTimer() - startTime > upgradeTimes[0])
         {
            upgradeTimes.shift();
            ++maxBalls;
         }
         while(--_loc1_ > -1)
         {
            _loc2_ = balls[_loc1_];
            if(_loc2_.body.m_position.y > TARGET_TOP)
            {
               if((_loc8_ = getColourAtPosition(_loc2_.body.m_position.x)) == _loc2_.colour)
               {
                  score += POINTS[pointsIndex];
                  showScoreIcon(POINTS[pointsIndex],_loc2_);
                  if(pointsIndex < POINTS.length - 1)
                  {
                     ++pointsIndex;
                  }
                  SoundManager.getInstance().getSoundFromID("ballCorrect").play();
                  SoundManager.getInstance().getSoundFromID("points").play();
               }
               else
               {
                  pointsIndex = 0;
                  if(_loc2_.colour == Ball.COLOUR_BLACK)
                  {
                     score += BLACK_PENALTY_POINTS;
                     showScoreIcon(BLACK_PENALTY_POINTS,_loc2_);
                     if(score < 0)
                     {
                        score = 0;
                     }
                     --numEvilBalls;
                     addToBadBandMembers([0,1,2,3]);
                     SoundManager.getInstance().getSoundFromID("blackBall").play();
                  }
                  else
                  {
                     score += PENALTY_POINTS;
                     if(score < 0)
                     {
                        score = 0;
                     }
                     showScoreIcon(PENALTY_POINTS,_loc2_);
                     addToBadBandMembers([_loc8_]);
                     SoundManager.getInstance().getSoundFromID("ballWrong").play();
                  }
               }
               balls.splice(_loc1_,1);
               world.DestroyBody(_loc2_.body);
               _loc2_.destroy();
            }
            else if(_loc2_ is EvilBall && (_loc2_ as EvilBall).dead)
            {
               balls.splice(_loc1_,1);
               world.DestroyBody(_loc2_.body);
               _loc2_.destroy();
               --numEvilBalls;
               SoundManager.getInstance().getSoundFromID("blackBallPop").play();
            }
         }
      }
      
      private function onClickMusicCredit(param1:MouseEvent) : void
      {
         var loader:Loader;
         var e:MouseEvent = param1;
         var urlRequest:URLRequest = new URLRequest(Main.NEOLICIOUS_LINK);
         try
         {
            navigateToURL(urlRequest,"_blank");
         }
         catch(e:Error)
         {
         }
         Tracker.kvClick(Main.MEMECOUNTER_NEOLICIOUS_TRACKING);
         loader = new Loader();
         loader.load(new URLRequest(Main.MUSIC_CREDIT_TRACKING));
      }
      
      private function onMouseDown(param1:MouseEvent) : void
      {
         drawing = true;
         mx = mouseX;
         my = mouseY;
      }
      
      private function onMouseUp(param1:MouseEvent) : void
      {
         drawing = false;
      }
      
      private function initWalls() : void
      {
         var _loc1_:b2BoxDef = null;
         var _loc2_:b2BodyDef = null;
         _loc1_ = new b2BoxDef();
         _loc1_.extents.Set(25,SCREEN_HEIGHT);
         _loc1_.friction = 0;
         _loc1_.restitution = 1;
         _loc2_ = new b2BodyDef();
         _loc2_.position.Set(-25,0);
         _loc2_.AddShape(_loc1_);
         world.CreateBody(_loc2_);
         _loc2_ = new b2BodyDef();
         _loc2_.position.Set(SCREEN_WIDTH + 25,0);
         _loc2_.AddShape(_loc1_);
         world.CreateBody(_loc2_);
      }
      
      public function getScore() : int
      {
         return score;
      }
      
      private function initWorld() : void
      {
         var _loc1_:b2AABB = new b2AABB();
         _loc1_.minVertex.Set(-1000,-3000);
         _loc1_.maxVertex.Set(1000,1000);
         var _loc2_:b2Vec2 = new b2Vec2(0,50);
         world = new b2World(_loc1_,_loc2_,true);
      }
      
      private function gameOver() : void
      {
         trace("GAME OVER!");
         removeEventListener(Event.ENTER_FRAME,onEnterFrame);
         SoundManager.getInstance().getSoundFromID("music").stop();
         dispatchEvent(new Event(Event.COMPLETE));
      }
      
      private function initTargetDividers() : void
      {
         var _loc1_:b2BoxDef = null;
         var _loc2_:b2CircleDef = null;
         var _loc3_:b2BodyDef = null;
         var _loc9_:int = 0;
         var _loc4_:Array = [14.5,20];
         var _loc5_:Number = 14.5;
         var _loc6_:Array = [[0,376],[137.5,376],[275.5,376],[413,376],[551,376]];
         var _loc7_:Array = [0,7];
         var _loc8_:Array = [0,-12.5];
         _loc1_ = new b2BoxDef();
         _loc1_.extents.Set(_loc4_[0],_loc4_[1]);
         _loc1_.localPosition.x = _loc7_[0];
         _loc1_.localPosition.y = _loc7_[1];
         _loc1_.friction = 0.1;
         _loc1_.restitution = 0.1;
         _loc2_ = new b2CircleDef();
         _loc2_.radius = _loc5_;
         _loc2_.localPosition.x = _loc8_[0];
         _loc2_.localPosition.y = _loc8_[1];
         _loc2_.friction = 0.1;
         _loc2_.restitution = 0.1;
         _loc9_ = 0;
         while(_loc9_ < _loc6_.length)
         {
            _loc3_ = new b2BodyDef();
            _loc3_.position.x = _loc6_[_loc9_][0];
            _loc3_.position.y = _loc6_[_loc9_][1];
            _loc3_.AddShape(_loc1_);
            _loc3_.AddShape(_loc2_);
            world.CreateBody(_loc3_);
            _loc9_++;
         }
      }
      
      private function getColourAtPosition(param1:Number) : int
      {
         var _loc2_:int = 0;
         if(param1 < RED_TARGET_MAX)
         {
            _loc2_ = Ball.COLOUR_RED;
         }
         else if(param1 < YELLOW_TARGET_MAX)
         {
            _loc2_ = Ball.COLOUR_YELLOW;
         }
         else if(param1 < BLUE_TARGET_MAX)
         {
            _loc2_ = Ball.COLOUR_BLUE;
         }
         else if(param1 < GREEN_TARGET_MAX)
         {
            _loc2_ = Ball.COLOUR_GREEN;
         }
         return _loc2_;
      }
      
      override public function destroy() : void
      {
         removeEventListener(Event.ENTER_FRAME,onEnterFrame);
      }
      
      private function addEvilBall() : void
      {
         balls.push(new EvilBall(world,ballSprite,assetFactory));
         ++numEvilBalls;
      }
   }
}
